#include "arduinoFFT.h"

// biblioteci pentru display-ul OLED
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

// biblioteci pentru matricea de leduri
#include <MD_MAX72xx.h>
#include <Max72xxPanel.h>

// biblioteca pentru banda de leduri
#include <Adafruit_NeoPixel.h>

#define N_PIXELS  20  // Numraul de pixeli de pe banda led
#define LED_PIN 3

Adafruit_NeoPixel strip = Adafruit_NeoPixel(N_PIXELS, LED_PIN, NEO_GRB + NEO_KHZ800);

#define SAMPLES 64 // numarul de citiri

#define SCREEN_WIDTH 128 // latimea displaylui
#define SCREEN_HEIGHT 32 // inaltimea displaylui

#define BAR_WIDTH 10 // latimea unei bari de frecventa din spectru
#define BARS 12 // numarul de bari de frecventa de pe display

int old_value[BARS];

#define ANALOG_PIN A0

// SSD1306 display conectat prin protocolul I2C
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, -1);

double value_freq[SAMPLES];
double value_read[SAMPLES];
unsigned long period;

// transformarea fourier a valorilor citite in spectrul de frecvente
arduinoFFT  FFT = arduinoFFT(value_read, value_freq, SAMPLES, 8000);

// definire matrice
#define HARDWARE_TYPE MD_MAX72XX::FC16_HW
#define DEVICES  4
#define CLK_PIN   13
#define DATA_PIN  11
#define CS_PIN    10
#define  MATRIX_WIDTH 32
#define  MATRIX_HEIGHT 8

char frequency_spec[MATRIX_WIDTH];
int last_val[MATRIX_WIDTH];
int MATRIX_VALUES[] = {0, 1, 3, 7, 15, 31, 63, 127, 255};
MD_MAX72XX mx = MD_MAX72XX(HARDWARE_TYPE, CS_PIN, DEVICES);



void setup() {

  // Setup display
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  display.clearDisplay();
  display.display();
  display.setRotation(0);
  display.invertDisplay(false);
  display.drawLine(0, 0, SCREEN_WIDTH, 0, SSD1306_WHITE);

  // perioada intre sampleuri
  period = 125;

  // Setup leds
  strip.begin();
  
  // Setup matrice
  mx.begin();
  delay(50);

}

void loop() {

  // citirea valorilor
  for (int i = 0; i < SAMPLES; i++) {
    unsigned long start = micros();
    int value = analogRead(ANALOG_PIN);
    value_read[i] = 0;
    // conditie pentru ignorarea zgomotelor
    if(value < 900)
      value_read[i] = value;
    value_freq[i] = 0;
    
    while (micros() < (start + period)) {
      yield();
    }
  }

    // transformarea in sepctrul de frecvente folosind FFT
    FFT.Windowing(value_read, SAMPLES, FFT_WIN_TYP_HAMMING, FFT_FORWARD);
    FFT.Compute(value_read, value_freq, SAMPLES, FFT_FORWARD);
    FFT.ComplexToMagnitude(value_read, value_freq, SAMPLES);

  display_i2c();
  display_matrix();
  display_leds();
}

uint32_t Wheel(byte WheelPos) {
  if(WheelPos < 85) {
   return strip.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
  } else if(WheelPos < 170) {
   WheelPos -= 85;
   return strip.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  } else {
   WheelPos -= 170;
   return strip.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  }
}

void display_leds(){
  int value = 0;
   for (int i = 0; i < SAMPLES; i++)
    {
      value += value_read[i];
    }
    value /= SAMPLES;
    value = constrain(value, 0, 32);
    value = map(value, 0, 32, 0, N_PIXELS);

   for(int i = 0; i < N_PIXELS; i++) {  
    if(i >= value)               
     strip.setPixelColor(i, 0, 0, 0);
    else 
     strip.setPixelColor(i, Wheel(map(i, 0, strip.numPixels()-1 , 30, 150)));
   }
   strip.show();
}

void display_i2c(){
    int step = SAMPLES / BARS;
    int count = 0;
    for (int i = 0; i < SAMPLES; i += step)
    {
      frequency_spec[count] = 0;
      for (int j = 0 ; j < step ; j++) {
        frequency_spec[count] += value_freq[i + j];
      }
      frequency_spec[count] /= step;
      count++;
    }

    for (int i = 0; i < BARS; i++)
    {
      frequency_spec[i] = constrain(frequency_spec[i], 0, 32);
      frequency_spec[i] = map(frequency_spec[i], 0, 32, 0, SCREEN_HEIGHT);
      int height = frequency_spec[i];
      
      int display_val = MATRIX_VALUES[height];
      int display_col = i;

      // stergerea valorii vechi de pe display
      display.fillRect(i * BAR_WIDTH + 6, 1, 7, old_value[i], SSD1306_BLACK);
      old_value[i] = height; 
      // afisarea noii valori pe display
      display.fillRect(i * BAR_WIDTH + 6, 1, 7, height, SSD1306_WHITE);
    }
    display.display(); 
}

void display_matrix(){
    int step = SAMPLES / MATRIX_WIDTH;
    int count = 0;
    for (int i = 0; i < SAMPLES; i += step)
    {
      frequency_spec[count] = 0;
      for (int j = 0 ; j < step ; j++) {
        frequency_spec[count] += value_freq[i + j];
      }
      frequency_spec[count] /= step;
      count++;
    }

    for (int i = 0; i < MATRIX_WIDTH; i++)
    {
      frequency_spec[i] = constrain(frequency_spec[i], 0, 32);
      frequency_spec[i] = map(frequency_spec[i], 0, 32, 0, MATRIX_HEIGHT);
      int height = frequency_spec[i];

      // ofera un efect vizual imbunatatit, cand scade frecventa de la o iteratie
      // la alta pe afisaz scade doar o unitate
      last_val[i] = last_val[i] - 1;
      if (height > last_val[i])
        last_val[i] = height ;
      height = last_val[i];
      
      int display_val = MATRIX_VALUES[height];
      int display_col = i;
      mx.setColumn(display_col, display_val);
    }
}
